@@ -6,6 +6,8 @@
           thin-restore-help
           thin-rmap-help
           thin-delta-help
+	  thin-metadata-pack-help
+	  thin-metadata-unpack-help
 
           cache-check-help
           cache-restore-help
@@ -22,11 +24,14 @@
   ;; These long string constants really confuse vim and mess up Paredit mode.
   ;; So moving into a separate file.
   (define thin-check-help
-    "Usage: thin_check [options] {device|file}
+          "Usage: thin_check [options] {device|file}
 Options:
   {-q|--quiet}
   {-h|--help}
   {-V|--version}
+  {-m|--metadata-snap}
+  {--auto-repair}
+  {--override-mapping-root}
   {--clear-needs-check-flag}
   {--ignore-non-fatal-errors}
   {--skip-mappings}
@@ -45,6 +50,9 @@ Options:
   {-h|--help}
   {-i|--input} <input xml file>
   {-o|--output} <output device or file>
+  {--transaction-id} <natural>
+  {--data-block-size} <natural>
+  {--nr-data-blocks} <natural>
   {-q|--quiet}
   {-V|--version}")
 
@@ -68,6 +76,36 @@ Options:
   {-h|--help}
   {-V|--version}")
 
+  (define thin-metadata-pack-help
+    "thin_metadata_pack 0.9.0-rc2
+Produces a compressed file of thin metadata.  Only packs metadata blocks that are actually used.
+
+USAGE:
+    thin_metadata_pack -i <DEV> -o <FILE>
+
+FLAGS:
+    -h, --help       Prints help information
+    -V, --version    Prints version information
+
+OPTIONS:
+    -i <DEV>         Specify thinp metadata binary device/file
+    -o <FILE>        Specify packed output file")
+
+  (define thin-metadata-unpack-help
+    "thin_metadata_unpack 0.9.0-rc2
+Unpack a compressed file of thin metadata.
+
+USAGE:
+    thin_metadata_unpack -i <DEV> -o <FILE>
+
+FLAGS:
+    -h, --help       Prints help information
+    -V, --version    Prints version information
+
+OPTIONS:
+    -i <DEV>         Specify thinp metadata binary device/file
+    -o <FILE>        Specify packed output file")
+
   (define cache-check-help
     "Usage: cache_check [options] {device|file}
 Options:
