@@ -16,15 +16,18 @@
 // with thin-provisioning-tools.  If not, see
 // <http://www.gnu.org/licenses/>.
 
-#include "thin-provisioning/emitter.h"
-#include "thin-provisioning/metadata_dumper.h"
-#include "thin-provisioning/mapping_tree.h"
 #include "persistent-data/data-structures/simple_traits.h"
 #include "persistent-data/file_utils.h"
+#include "persistent-data/space-maps/noop.h"
+#include "thin-provisioning/emitter.h"
+#include "thin-provisioning/mapping_tree.h"
+#include "thin-provisioning/metadata_dumper.h"
 
+#include <algorithm>
 #include <map>
 #include <vector>
 
+using namespace boost;
 using namespace persistent_data;
 using namespace thin_provisioning;
 
@@ -121,12 +124,19 @@ namespace {
 		set<uint32_t> dd_;
 	};
 
-	set<uint32_t>
+	// See comment on get_map_ids
+	optional<set<uint32_t> >
 	get_dev_ids(transaction_manager &tm, block_address root) {
 		d_thin_id_extractor de;
 		fatal_details_damage dv;
 		auto tree = device_tree(tm, root, device_tree_detail::device_details_traits::ref_counter());
-		walk_device_tree(tree, de, dv);
+
+		try {
+			walk_device_tree(tree, de, dv, true);
+		} catch (...) {
+			return optional<set<uint32_t>>();
+		}
+
 		return de.dd_;
 	}
 
@@ -137,13 +147,21 @@ namespace {
 
 		set<uint32_t> dd_;
 	};
-	
-	set<uint32_t>
+
+	// The walk will do more sanity checks than we did when scanning the metadata, so
+	// it's possible that it will fail and throw a metadata damage exception.
+	optional<set<uint32_t> >
 	get_map_ids(transaction_manager &tm, block_address root) {
 		m_thin_id_extractor me;
 		fatal_mapping_damage mv;
 		auto tree = dev_tree(tm, root, mapping_tree_detail::mtree_traits::ref_counter(tm));
-		walk_mapping_tree(tree, me, mv);
+
+		try {
+			walk_mapping_tree(tree, me, mv, true);
+		} catch (...) {
+			return optional<set<uint32_t>>();
+		}
+
 		return me.dd_;
 	}
 }
@@ -214,8 +232,8 @@ namespace {
 
 		block_address b;
 		unsigned values;
-		uint64_t key_low;
-		uint64_t key_high;
+		::uint64_t key_low;
+		::uint64_t key_high;
 		//set<uint32_t> devices;
 		uint32_t age;
 		map<uint32_t, uint32_t> time_counts;
@@ -258,13 +276,19 @@ namespace {
 
 	class gatherer {
 	public:
-		gatherer(block_manager<> &bm)
+		gatherer(block_manager &bm)
 			: bm_(bm),
 			  referenced_(bm.get_nr_blocks(), false),
 			  examined_(bm.get_nr_blocks(), false) {
 		}
 
-		optional<pair<block_address, block_address>>
+		struct roots {
+			block_address mapping_root;
+			block_address detail_root;
+			uint32_t time;
+		};
+
+		optional<roots>
 		find_best_roots(transaction_manager &tm) {
 			vector<node_info> mapping_roots;
 			vector<node_info> device_roots;
@@ -280,11 +304,13 @@ namespace {
 				auto info = get_info(b);
 
 				if (info.valid) {
-					if (info.type == TOP_LEVEL)
+					if (info.type == TOP_LEVEL) {
 						mapping_roots.push_back(info);
+					}
 
-					else if (info.type == DEVICE_DETAILS)
+					else if (info.type == DEVICE_DETAILS) {
 						device_roots.push_back(info);
+					}
 				}
 			}
 
@@ -306,12 +332,27 @@ namespace {
 #endif
 
 			if (pairs.size())
-				return pairs[0];
+				return mk_roots(pairs[0]);
 			else
-				return optional<pair<block_address, block_address>>();
+				return optional<roots>();
 		}
 
 	private:
+		uint32_t get_time(block_address b) const {
+			auto i = lookup_info(b);
+			return i ? i->age : 0;
+		}
+
+		roots mk_roots(pair<block_address, block_address> const &p) {
+			roots r;
+
+			r.mapping_root = p.second;
+			r.detail_root = p.first;
+			r.time = max<block_address>(get_time(p.first), get_time(p.second));
+
+			return r;
+		}
+
 		bool set_eq(set<uint32_t> const &lhs, set<uint32_t> const &rhs) {
 			for (auto v : lhs)
 				if (!rhs.count(v))
@@ -340,12 +381,18 @@ namespace {
 			sort(pairs.begin(), pairs.end(), cmp_time_counts);
 
 			map<block_address, set<uint32_t>> ds;
-			for (auto b : d_roots)
-				ds.insert(make_pair(b, get_dev_ids(tm, b)));
+			for (auto b : d_roots) {
+				auto maybe_ids = get_dev_ids(tm, b);
+				if (maybe_ids)
+					ds.insert(make_pair(b, *maybe_ids));
+			}
 
 			map<block_address, set<uint32_t>> ms;
-			for (auto b : m_roots)
-				ms.insert(make_pair(b, get_map_ids(tm, b)));
+			for (auto b : m_roots) {
+				auto maybe_ids = get_map_ids(tm, b);
+				if (maybe_ids)
+					ms.insert(make_pair(b, *maybe_ids));
+			}
 
 			// now we check that the thin_ids are identical
 			vector<pair<block_address, block_address>> filtered;
@@ -424,13 +471,13 @@ namespace {
 			}
 		}
 
-		node_info get_internal_info(block_manager<>::read_ref &rr) {
+		node_info get_internal_info(block_manager::read_ref &rr) {
 			node_info info;
 			info.b = rr.get_location();
 			 
 			// values refer to blocks, so we should have infos for them.
 			auto n = to_node<block_traits>(rr);
-			uint64_t key_low = 0;
+			::uint64_t key_low = 0;
 			unsigned values = 0;
 
 			for (unsigned i = 0; i < n.get_nr_entries(); i++) {
@@ -473,7 +520,7 @@ namespace {
 			return info;
 		}
 
-		node_info get_leaf_info(block_manager<>::read_ref &rr, node_header const &hdr) {
+		node_info get_leaf_info(block_manager::read_ref &rr, node_header const &hdr) {
 			node_info info;
 			info.b = rr.get_location();
 
@@ -494,7 +541,7 @@ namespace {
 					info.nr_mappings += n.value_at(i).mapped_blocks_;
 				}
 
-			} else if (vsize == sizeof(uint64_t)) {
+			} else if (vsize == sizeof(::uint64_t)) {
 				auto n = to_node<uint64_traits>(rr);
 
 				if (n.get_nr_entries()) {
@@ -567,8 +614,8 @@ namespace {
 
 				return it->second;
 			} else {
-				node_info info = get_info_(b);
 				examined_[b] = true;
+				node_info info = get_info_(b);
 				if (!failed(info))
 					infos_.insert(make_pair(b, info));
 
@@ -576,7 +623,15 @@ namespace {
 			}
 		}
 
-		block_manager<> &bm_;
+		optional<node_info> lookup_info(block_address b) const {
+			auto it = infos_.find(b);
+			if (it == infos_.end())
+				return optional<node_info>();
+
+			return optional<node_info>(it->second);
+		}
+
+		block_manager &bm_;
 		vector<bool> referenced_;
 		vector<bool> examined_;
 		map<block_address, node_info> infos_;
@@ -586,14 +641,14 @@ namespace {
 //----------------------------------------------------------------
 
 namespace {
-	class mapping_emitter : public mapping_tree_detail::mapping_visitor {
+	class mapping_emit_visitor : public mapping_tree_detail::mapping_visitor {
 	public:
-		mapping_emitter(emitter::ptr e)
+		mapping_emit_visitor(emitter::ptr e)
 			: e_(e),
 			  in_range_(false) {
 		}
 
-		~mapping_emitter() {
+		~mapping_emit_visitor() {
 			end_mapping();
 		}
 
@@ -603,7 +658,7 @@ namespace {
 		}
 
 	private:
-		void start_mapping(uint64_t origin_block, block_time const &bt) {
+		void start_mapping(::uint64_t origin_block, block_time const &bt) {
 			origin_start_ = origin_block;
 			dest_start_ = bt.block_;
 			time_ = bt.time_;
@@ -622,7 +677,7 @@ namespace {
 			}
 		}
 
-		void add_mapping(uint64_t origin_block, block_time const &bt) {
+		void add_mapping(::uint64_t origin_block, block_time const &bt) {
 			if (!in_range_)
 				start_mapping(origin_block, bt);
 
@@ -645,15 +700,15 @@ namespace {
 		bool in_range_;
 	};
 
-	class mapping_tree_emitter : public mapping_tree_detail::device_visitor {
+	class mapping_tree_emit_visitor : public mapping_tree_detail::device_visitor {
 	public:
-		mapping_tree_emitter(dump_options const &opts,
-				     metadata::ptr md,
-				     emitter::ptr e,
-				     dd_map const &dd,
-				     mapping_tree_detail::damage_visitor::ptr damage_policy)
+		mapping_tree_emit_visitor(dump_options const &opts,
+					  transaction_manager &tm,
+					  emitter::ptr e,
+					  dd_map const &dd,
+					  mapping_tree_detail::damage_visitor::ptr damage_policy)
 			: opts_(opts),
-			  md_(md),
+			  tm_(tm),
 			  e_(e),
 			  dd_(dd),
 			  damage_policy_(damage_policy) {
@@ -693,32 +748,131 @@ namespace {
 		}
 
 	private:
-		void emit_mappings(uint64_t dev_id, block_address subtree_root) {
-			mapping_emitter me(e_);
-			single_mapping_tree tree(*md_->tm_, subtree_root,
-						 mapping_tree_detail::block_time_ref_counter(md_->data_sm_));
-			walk_mapping_tree(tree, dev_id, static_cast<mapping_tree_detail::mapping_visitor &>(me), *damage_policy_);
+		void emit_mappings(::uint64_t dev_id, block_address subtree_root) {
+			mapping_emit_visitor me(e_);
+
+			// Since we're not mutating the btrees we don't need a real space map
+			noop_map::ptr sm(new noop_map);
+			single_mapping_tree tree(tm_, subtree_root, mapping_tree_detail::block_time_ref_counter(sm));
+			walk_mapping_tree(tree, dev_id, static_cast<mapping_tree_detail::mapping_visitor &>(me), *damage_policy_, true);
 		}
 
 		dump_options const &opts_;
-		metadata::ptr md_;
+		transaction_manager &tm_;
 		emitter::ptr e_;
 		dd_map const &dd_;
 		mapping_tree_detail::damage_visitor::ptr damage_policy_;
 	};
 
-	block_address get_nr_blocks(metadata::ptr md) {
-		if (md->data_sm_)
-			return md->data_sm_->get_nr_blocks();
+	block_address get_nr_blocks(metadata &md) {
+		if (md.data_sm_)
+			return md.data_sm_->get_nr_blocks();
 
-		else if (md->sb_.blocknr_ == superblock_detail::SUPERBLOCK_LOCATION)
+		else if (md.sb_.blocknr_ == superblock_detail::SUPERBLOCK_LOCATION)
 			// grab from the root structure of the space map
-			return get_nr_blocks_in_data_sm(*md->tm_, &md->sb_.data_space_map_root_);
+			return get_nr_blocks_in_data_sm(*md.tm_, &md.sb_.data_space_map_root_);
 
 		else
 			// metadata snap, we really don't know
 			return 0ull;
 	}
+
+	void
+	emit_trees_(block_manager::ptr bm, superblock_detail::superblock const &sb,
+                    emitter::ptr e, override_options const &ropts)
+	{
+		metadata md(bm, sb);
+		dump_options opts;
+		details_extractor de(opts);
+		device_tree_detail::damage_visitor::ptr dd_policy(details_damage_policy(true));
+		walk_device_tree(*md.details_, de, *dd_policy, true);
+
+		e->begin_superblock("", sb.time_,
+				    sb.trans_id_,
+				    sb.flags_,
+				    sb.version_,
+				    sb.data_block_size_,
+				    get_nr_blocks(md),
+				    boost::optional<block_address>());
+
+		{
+			mapping_tree_detail::damage_visitor::ptr md_policy(mapping_damage_policy(true));
+			mapping_tree_emit_visitor mte(opts, *md.tm_, e, de.get_details(), mapping_damage_policy(true));
+			walk_mapping_tree(*md.mappings_top_level_, mte, *md_policy, true);
+		}
+
+		e->end_superblock();
+	}
+
+	void
+	find_better_roots_(block_manager::ptr bm, superblock_detail::superblock &sb)
+	{
+		// We assume the superblock is wrong, and find the best roots
+		// for ourselves.  We've had a few cases where people have
+		// activated a pool on multiple hosts at once, which results in
+		// the superblock being over written.
+		gatherer g(*bm);
+		auto tm = open_tm(bm, superblock_detail::SUPERBLOCK_LOCATION);
+		auto p = g.find_best_roots(*tm);
+
+		if (p) {
+			sb.metadata_snap_ = 0;
+			sb.time_ = p->time;
+			sb.device_details_root_ = p->detail_root;
+			sb.data_mapping_root_ = p->mapping_root;
+			sb.metadata_nr_blocks_ = bm->get_nr_blocks();
+		}
+	}
+
+
+
+	superblock_detail::superblock
+	recreate_superblock(override_options const &opts)
+	{
+		superblock_detail::superblock sb;
+		memset(&sb, 0, sizeof(sb));
+
+		// FIXME: we need to get this by walking both the mapping and device trees.
+		sb.time_ = 100000;
+
+
+		sb.trans_id_ = opts.get_transaction_id();
+		sb.version_ = superblock_detail::METADATA_VERSION;
+		sb.data_block_size_ = opts.get_data_block_size();
+
+		// Check that this has been overridden.
+		opts.get_nr_data_blocks();
+
+		return sb;
+	}
+
+	optional<superblock_detail::superblock>
+	maybe_read_superblock(block_manager::ptr bm)
+	{
+		try {
+			auto sb = read_superblock(bm);
+			return optional<superblock_detail::superblock>(sb);
+		} catch (...) {
+		}
+
+		return optional<superblock_detail::superblock>();
+	}
+
+	void
+	metadata_repair_(block_manager::ptr bm, emitter::ptr e, override_options const &opts)
+	{
+		auto msb = maybe_read_superblock(bm);
+		if (!msb)
+			msb = recreate_superblock(opts);
+
+		auto tm = open_tm(bm, superblock_detail::SUPERBLOCK_LOCATION);
+
+		if (!get_dev_ids(*tm, msb->device_details_root_) ||
+	            !get_map_ids(*tm, msb->data_mapping_root_))
+			find_better_roots_(bm, *msb);
+
+		emit_trees_(bm, *msb, e, opts);
+	}
 }
 
 //----------------------------------------------------------------
@@ -728,94 +882,49 @@ thin_provisioning::metadata_dump(metadata::ptr md, emitter::ptr e, dump_options
 {
 	details_extractor de(opts);
 	device_tree_detail::damage_visitor::ptr dd_policy(details_damage_policy(false));
-	walk_device_tree(*md->details_, de, *dd_policy);
+	walk_device_tree(*md->details_, de, *dd_policy, true);
 
 	e->begin_superblock("", md->sb_.time_,
 			    md->sb_.trans_id_,
 			    md->sb_.flags_,
 			    md->sb_.version_,
 			    md->sb_.data_block_size_,
-			    get_nr_blocks(md),
+			    get_nr_blocks(*md),
 			    boost::optional<block_address>());
 
 	{
 		mapping_tree_detail::damage_visitor::ptr md_policy(mapping_damage_policy(false));
-		mapping_tree_emitter mte(opts, md, e, de.get_details(), mapping_damage_policy(false));
-		walk_mapping_tree(*md->mappings_top_level_, mte, *md_policy);
+		mapping_tree_emit_visitor mte(opts, *md->tm_, e, de.get_details(), mapping_damage_policy(false));
+		walk_mapping_tree(*md->mappings_top_level_, mte, *md_policy, true);
 	}
 
 	e->end_superblock();
 }
 
 void
-thin_provisioning::metadata_repair(block_manager<>::ptr bm, emitter::ptr e)
+thin_provisioning::metadata_repair(block_manager::ptr bm, emitter::ptr e, override_options const &opts)
 {
-	// We assume the superblock is wrong, and find the best roots
-	// for ourselves.  We've had a few cases where people have
-	// activated a pool on multiple hosts at once, which results in
-	// the superblock being over written.
+	try {
+		metadata_repair_(bm, e, opts);
 
-
-	gatherer g(*bm);
-	auto tm = open_tm(bm, superblock_detail::SUPERBLOCK_LOCATION);
-	auto p = g.find_best_roots(*tm);
-
-	metadata::ptr md;
-
-	if (p) {
-		// We found good roots, so we fill out our own superblock,
-		// with some help from the old sb.
-		 
-		// FIXME: what happens if the superblock can't be read?
-		// catch and fill out defaults?  what should the data_block_size be?
-		auto sb = read_superblock(*bm);
-
-		sb.metadata_snap_ = 0;
-
-		sb.device_details_root_ = p->first;
-		sb.data_mapping_root_ = p->second;
-		sb.metadata_nr_blocks_ = bm->get_nr_blocks();
-
-		md.reset(new metadata(bm, sb));
-
-	} else {
-		// We couldn't find any good roots, so we'll fall back to using the
-		// on disk superblock.
-		md.reset(new metadata(bm, false));
+	} catch (override_error const &e) {
+		ostringstream out;
+		out << "The following field needs to be provided on the command line due to corruption in the superblock: "
+		    << e.what();
+		throw runtime_error(out.str());
 	}
-
-	dump_options opts;
-	details_extractor de(opts);
-	device_tree_detail::damage_visitor::ptr dd_policy(details_damage_policy(false));
-	walk_device_tree(*md->details_, de, *dd_policy);
-
-	e->begin_superblock("", md->sb_.time_,
-			    md->sb_.trans_id_,
-			    md->sb_.flags_,
-			    md->sb_.version_,
-			    md->sb_.data_block_size_,
-			    get_nr_blocks(md),
-			    boost::optional<block_address>());
-
-	{
-		mapping_tree_detail::damage_visitor::ptr md_policy(mapping_damage_policy(true));
-		mapping_tree_emitter mte(opts, md, e, de.get_details(), mapping_damage_policy(false));
-		walk_mapping_tree(*md->mappings_top_level_, mte, *md_policy);
-	}
-
-	e->end_superblock();
 }
 
 //----------------------------------------------------------------
 
 void
-thin_provisioning::metadata_dump_subtree(metadata::ptr md, emitter::ptr e, bool repair, uint64_t subtree_root) {
-	mapping_emitter me(e);
+thin_provisioning::metadata_dump_subtree(metadata::ptr md, emitter::ptr e, bool repair, ::uint64_t subtree_root) {
+	mapping_emit_visitor me(e);
 	single_mapping_tree tree(*md->tm_, subtree_root,
 				 mapping_tree_detail::block_time_ref_counter(md->data_sm_));
 	// FIXME: pass the current device id instead of zero
 	walk_mapping_tree(tree, 0, static_cast<mapping_tree_detail::mapping_visitor &>(me),
-			  *mapping_damage_policy(repair));
+			  *mapping_damage_policy(repair), true);
 }
 
 //----------------------------------------------------------------
